@@ -62,19 +62,31 @@ Settings::Settings()
         if (homeBgFile)
         {
             char line[1024];
-            static const char *prefix1 = "BackgroundImage=file://";
-            static const char *prefix2 = "BackgroundImage=";
+            static const char *prefix1 = "BackgroundImage=";
+            static const char *prefix3 = "CachedAs=";
             while (fgets(line, sizeof(line), homeBgFile) != 0)
             {
                 if (strstr(line, prefix1) == line)
                 {
-                    _backgroundFilename = strndup(&line[strlen(prefix1)], strcspn(&line[strlen(prefix1)], "\n"));
-                    break;
+                    char tmp[1024];
+                    strncpy(
+                        tmp,
+                        &line[strlen(prefix1)],
+                        strcspn(
+                            &line[strlen(prefix1)],
+                            "\n"
+                        )
+                    );
+                    _backgroundFilename = urldecode(tmp);
                 }
-                else if (strstr(line, prefix2) == line)
+                else if (strstr(line, prefix3) == line)
                 {
-                    _backgroundFilename = strndup(&line[strlen(prefix2)], strcspn(&line[strlen(prefix2)], "\n"));
-                    break;
+                    int valuelen = strcspn(&line[strlen(prefix3)], "\n");
+                    if (valuelen > 0)
+                    {
+                        free(_backgroundFilename);
+                        _backgroundFilename = strndup(&line[strlen(prefix3)], valuelen);
+                    }
                 }
             }
 
@@ -95,6 +107,51 @@ Settings::~Settings()
     free(_borderColor);
 }
 
+
+
+int hexdigit(char hex)
+{
+    if (hex >= '0' && hex <= '9')
+        return hex - '0';
+
+    if (hex >= 'A' && hex <= 'F')
+        return hex - 'A' + 0xA;
+
+    if (hex >= 'a' && hex <= 'f')
+        return hex - 'a' + 0xA;
+
+    return 0;
+}
+
+
+
+char* Settings::urldecode(const char *url)
+{
+    if (strstr(url, "file://") == url)
+        url += 7;
+
+    char *result = (char*)malloc(strlen(url));
+    char *r = result;
+
+    while (*url)
+    {
+        if (*url == '%')
+        {
+            ++url;
+            int val = hexdigit(*url++);
+            val <<= 4;
+            val |= hexdigit(*url++);
+            *r++ = val;
+        }
+        else
+            *r++ = *url++;
+    }
+
+    return result;
+}
+
+
+
 #define SPACECHARS  " \t"
 
 void Settings::parseLine(const char *line)
